function [Xinv, tvec, xmesh, ymesh, chi2, lambda, ...
          FastPtot, FastPbelow, FastPxpoint, good] = ...
          fabcat(shot,tmin,tmax,points)

% function [Xinv, tvec, xmesh, ymesh, chi2, lambda, ...
%           FastPtot, FastPbelow, FastPxpoint, good] = ...
%           fabcat(shot,tmin,tmax,points)
%
% Algorithm for fast tomography reconstruction for bolometry.
% Package Fast-Algorithm Bolometric Computer Aided Tomography (FABCAT).
% Only one inversion matrix M is calculated per shot.
% If the tmin, tmax times were not specified, all LIUQE times are applied.
% Otherwise all (normal) bolometry time samples within tmin, tmax are
% reconstructed. Points limits the number of timeframes in the reconstruction.
%
% Definition of the X-point:
% The pixel with highest emissivity and its upper, lower and right adjacent
% pixels within pixels 3 to 16 and 31 to 44 (the inferior part of two left
% border columns, the so-called potential X-point region) is are searched 
% and excluded.
% ATTENTION : THESE DEFINITIONS DEPEND ON THE CHOSEN PIXEL GRID !!! 
%
% See file FABCAT.pdf for further documentation (available on http://crpplocal).
%
% The results can be reached via global variables:
%
%  Y            : the processed (calibrated) bolometry signals
%  tvec         : the vector of times
%  Xinv         : 3-dim emissivity (ny x nx x tvec) reconstruction image [W/m^3]  
%  xmesh, ymesh : geometrical setup of the pixels [cm]
%  chi2, lambda : resulting chi^2 and smoothing factor
%                 length(chi2)=length(tvec)
%  FastPtot     : Radiated power inside the whole tokamak [W]
%  FastPxpoint  : Radiated power coming from the x-point [W]
%  FastPbelow   : Radiated power coming from the divertor region [W]
%                 All radiated power variables have length(tvec)
%  good         : Flag indicating if the reconstruction was successful
%
% History:        
%
%  Christian Schlatter, TPIV CRPP EPFL, February 2002 (first release)
%  
% Final release version 1.4 (2002/02/07)
 
global Y tvec Xinv chi2 lambda xmesh ymesh errflag
global FastPtot FastPbelow FastPxpoint good

disp(' ') 
disp('Fast-Algorithm Bolometric Computer Aided Tomography (FABCAT).');
disp('Final release 1.4')
disp('Latest modification on 2002/02/07.') 
disp(' '); 

debut=cputime;

% User adjustable parameters **************************************************

f_cam       =        [1 1 1 1 1 1 1 1];
                   % Camera switch. 0 = disabled; 1 = enabled (default).

chantest    = 1;   % 0 : no testing for weird channels.
                   % 1 : testing using Christian Deschenaux's method (default)
                   %     based on backfiltering of the bolometer signals.
                   % 2 : testing using Arno Refke's method
                   %     based on a consistency check of the bolometer signals.
                   % 3 : testing using both methods.

smoothmode  = 2;   % 0 : no data processing.
		                 % 1 : Bernard Joye's data treatement, based on fitting of
                   %     a polynomial of second degree to the signals (slow).
                   % 2 : Data treatment using Christian Deschenaux's method
                   %     based on backfiltering of the raw signals (default).
                   % All data processing methods calibrate the signal in respect
                   % to the bolometer thermal inertia coefficents TAU.

delchanuser = [];  % User defined channels to exclude from the reconstruction.
                 
runsigma    = 3;   % Errorbar (for the signals) multiplication factor.
                   % The final errorbars are still multiplied by the
                   % bolometer etendue. Default = 3 %

lambda_init = 0.1; % Initial lambda for the minimum fisher loop (default = 0.1).
 
lamlim      = .01; % Minimum lambda for a good shot (default = 0.01).

chitar      = 1;   % Target value of chi^2. Default = 1

errchi      = .05; % Supremum of the difference between the time averaged final
                   % chi^2 and the target chi^2 defined by the flag CHITAR.
                   % Default = 0.05

loopfisher  = 3;   % Number of loops of the minimum fisher algorithm to
                   % minimize the Fisher information (default = 3).

iterfisher  = 4;   % Number of iterations of the minimum fisher algorithm
                   % to find the lambda that pushes chi^2 to chitar 
                   % (default = 4).
                   
limit       = 5;   % limiting factor for post-reconstruction channel
                   % consistency check (default = 5).

i_bord      = 0;   % 0 : Emissivity contribution coming from the vessels
                   %     border pixels are taken in consideration (default).
                   % 1 : emission form border pixels is set to zero.

x_blind     = 0;   % 1 : Excludes the expanded X-point from the reconstruction 
                   %     to increase the contrast of the reconstructed image.
                   %     For the definition of the X-point see flag CUTOFF.
                   % 0 : no X-point exclusion (default).
                   
cutoff      = 0;   % 0 : Automatic mode (default).
                   %     The X-point is searched automatically (see description
                   %     in the header of this file) and finally expanded to 
                   %     pixels in the potential X-point region having
                   %     emissivities higher than the percentage defined by
                   %     the flag LEVEL in respect to the X-point.
                   % 1 : Pixel 12 and 13 as expanded X-point.
                   % 2 : Pixel 10 to 16 as expanded X-point.
                   % 3 : Pixel 6 to 18 and 41 as expanded X-point.
                   % 4 : Pixel 5 to 20 and 40 to 43 as expanded X-point.
                   
level       = 80;  % Percentage of the X-points emissivity to establish the
                   % expanded X-point (CUTOFF has to be = 0). Default = 80 [%]. 

radiation   = 0;   % 1 : The fractions of radiation inside and outside of the
                   %     LCFS are determined using the slow fabpower.m
                   %     routine (originally the btboloti_pradinout.m routine).
                   % 0 : using the fast algorithm to determine FastPtot,
                   %     FastPxpoint and FastPbelow (see header of this file).

saving      = 0;   % 1 : Stores the variables shot, Xinv, tvec, FastPtot,
                   %     FastPbelow, FastPxpoint and good to the harddisk in
                   %     files called FABCAT_shotnumber.mat. Use the flag
                   %     SAVEPATH to chose the destination directory.
                   % 0 : no saving to harddisk (default).

savepath    =        '/home/tp47/matlab/FBTR/';
                   % Destination folder for the FABCAT_shotnumber.mat files.
                   % Make sure that you have writing priviliges for the chosen
                   % destination folder. See flag SAVING for more details. 

MDS         = 0;   % 1 : The results of the reconstruction are saved to the
                   %     MDS-tree. Make sure you have MDS writing priviliges.
                   % 0 : feature disabled (default).
                   
plotting    = 0;   % 1 : shows a contour plot of the reconstructed emissivity
                   %     for the last timeframe in the shot.
                   % 0 : no plotting (default).

% *****************************************************************************

errflag = 0;
good    = 0;

try

  close('Emissivity matrix for the last timeframe [W/m^3]');
   
end  

if nargin == 1     % LIUQE time mode for a single shot.

  tmin=NaN;
  tmax=NaN;

end

if nargin == 0     % LIUQE time mode for multiple shots.

  tmin=NaN;
  tmax=NaN;
  inputfile=input('Please provide the name of the shot array : ','s');
   
  try

    load(inputfile);
    disp('The file has successfully been loaded.');

  catch

    disp('The file doesn''t exist. Try another filename.');
    return

  end
   
  if ~exist('shot') % Test if the provided file is a reliable shot array

    disp('This is not a shot array.'); 
    disp('There wasn''t a variable called shot inside.');
    return

  end

  firstshot = shot(1);
  lastshot = shot(length(shot));
  clear shot;
  
else
  
  firstshot = shot;
  lastshot = shot;
  clear shot;

end

if nargin < 3
 
   tmax = NaN;
   
end

if nargin < 4       % Normal time mode without rescaling of the timeframes. 

   points=NaN;

end

for shot = firstshot : lastshot
  
  % FIRST STEP : GETTING THE SIGNALS
  
  Y = [];
  tvec = [];
  goodchans = [];
  errstr = [];

  [Y, tvec, errflag, errstr, goodchans] = ...
        fabsig(shot, f_cam, delchanuser, smoothmode, ...
		             tmin, tmax, points, chantest);

  % FABSIG.m gets the signals from the MDS tree, checks for weird channels and
  % does data processing (calibration).
  % If TMIN, TMAX were not specified, sets TVEC to nearest LIUQE times. 
  % Otherwise reads the normal times from the MDS tree.
  % If POINTS was submitted, the number or timeframes is rescaled to POINTS
  % frames.

  if errflag        % Shows error message when a problem occured calling FABSIG

    disp('  ');
    disp('  ');
    disp('  ');
    disp(errstr);
    return

  end
 
  % SECOND STEP : CHOOSING GEOMETRY
 
  load fabt         % Saved by the optional routine FABSETT.m. Includes
                    % NX, NY (number of pixels); XMIN, XMAX, YMIN, YMAX
                    % (extremal pixel positions); DX, DY (pixel extensions);
                    % XMESH, YMESH (pixels center coordinates); NUMDET
                    % (camera indicator) and T (the T-matrix).

  [nt, nl] = size(Y);  
                    
                    % NT: number of timeframes.
                    % NL: number of enabled channels.

  [usedch, igoodchans, inumdet] = intersect(goodchans, numdet);
                            
                    % Determines the channels which are good and activated.
                    % IGOODCHANS and INUMDET give the index
			                 % of the elements which are common.
		    	
  % THIRD STEP : DETERMING THE ERRORBARS.
  	  
  runsigma = runsigma / 100;
  
                    % change to [%].

  dY = runsigma*ones(64,1);
  
                    % Calculation of DY, the 'relative' error of the signals.

  load fabetendue   % Contains the etende of the bolometers in srad*cm^2.
                    % Calculated using FABSETETENDUE.m.

  angfact = 1 ./ etend';
         
  wdy = angfact ./ min(angfact);  
  
                    % Weighting matrix for the errorbars.
		
  dY = dY .* wdy;		 % Errorbars.

  Y = Y';	          % because of: size(framedata) = (tvec x goodchans)
                    % but you need size(Y) = (goodchans x tvec).

  % FOURTH STEP : EMISSIVITY RECONSTRUCTION.
  
  lambda = [];
  chim = [];
  Xinv = [];
  chi2 = [];

  xpoint = 1;       % The X-point hasn''t been excluded yet.

  [Xinv, lambda, chim, inumdet] = ...
      fabrec(Y, dY, xmesh, ymesh, tvec, inumdet, igoodchans, ...
              T ,lambda_init, loopfisher, iterfisher, i_bord, limit);

  % FABREC.m is doing the inversion. The reconstructed image is delivered
  % by Xinv. Final smoothing factor LAMBDA and chi2 (CHIM) are returned.
  % The routine is checking again for weird channels using a post-
  % reconstruction test on chi^2 (see flag LIMIT).

  Xinv=Xinv*100;    % Conversion to [W/m^3].
 
  if plotting & ~x_blind       
                    
                    % Shows a contour plot of the found emissivity of the
                    % first inversion if no second inversion is planned.
   
    figure('Name','Emissivity matrix for the last timeframe [W/m^3]');
    contourf(xmesh/100, ymesh/100, Xinv(:,:,length(tvec)), 50), ...
                     shading flat, axis equal
    colorbar;
    
  end
  
  % FIFTH STEP : RECONSTRUCTION RELIABILITY TESTING
  
  chi2 = mean(chim); % chi^2 averaged over all timeframes.

  disp('The inversion has finished');

  if lambda < lamlim | abs(chi2 - 1) > errchi
   
                     % Test if shot reconstruction is acceptable.

    disp(' ');
    result = ['Results: lambda=' num2str(lambda) ', mean(chi2)=' num2str(chi2)];
    disp(result);
    disp(['WARNING: reconstruction seems to be wrong in shot ' ...
                   int2str(shot) ' !']);
    disp(' ');
    errflag = 1;

  else  

    disp('  ');
    result = ['shot ',int2str(shot),' OK. (lambda = ', num2str(lambda), ...
                    ', chi2 = ', num2str(chi2),')'];
    disp(result);
    disp(' ');
    good = 1;

  end

  % SIXTH STEP : QUEST FOR THE X-POINT.
  
  X = reshape(Xinv, ny * nx, length(tvec));
  [K,J] = find(X == max(max(X)));
                    
                    % K : Pixel containing the X-point.
  
  Q = union(3:16,31:44);
  
                    % Q : Potential X-point region.

  K = intersect(K,Q);
  
                    % Limiting the X-point to the potential X-point region.

  if x_blind        % Optional X-point exclusion

    disp('  You have chosen to exlude the X-point');
    disp('  Running a second inversion...');
    disp('  ');
    
    xpoint = 0;

    if isempty(K)   % When the point with highest emissivity lies autside the
                    % potential X-point region.
                    
      disp('Identification of X-point impossible');
      xpoint = 1;
    
    else
    
      switch cutoff % Determination of the expanded X-point (multiple pixels)
  
        case 0
    
          [pixel, J] = find(X > (level / 100) * max(max(X)));
          
                    % LEVEL specifies the inferior radiation percentage limit
                    % for pixels participating to the expanded X-point.
                     
          pixel = intersect(pixel, Q);
       
        case 1       % Overrides the automatic X-point determination.
    
          pixel = [12, 13];
  
        case 2
   
          pixel = [10, 11, 12, 13, 14, 15, 16];
    
        case 3
   
          pixel = [6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 41];
   
        case 4
  
          pixel = [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ...
                     20, 40, 41, 42, 43];
  
      end

      g = reshape(Xinv / 100, ny * nx, length(tvec));

      contX = T(inumdet, pixel) * g(pixel, :);
      
                    % Contribution of the expanded X-point to the chord
                    % brightness Y.

      Y = Y - contX;
      
                    % New chord brightness without emissivity from the expanded
                    % X-point.

      [Xinv, lambda, chim, inumdet] = ...
          fabrec(Y, dY, xmesh, ymesh, tvec, inumdet, igoodchans, ...
                  T ,lambda_init, loopfisher, iterfisher, i_bord, limit);
                  
      % Second tomography inversion without expanded X-point contributions. 

      Xinv=Xinv*100;    % Conversion to [W/m^3]
      chi2=mean(chim);  % chi^2 averaged over all timeframes.

      if plotting       % Shows a contour plot of the found emissivity.
   
        figure('Name','Emissivity matrix for the last timeframe [W/m^3]');
        contourf(xmesh/100, ymesh/100, Xinv(:,:,length(tvec)), 50), ...
                     shading flat, axis equal
        colorbar;
        
      end

      disp('The second inversion has finished');

      if lambda < lamlim | abs(chi2 - 1) > errchi
   
                     % Test if shot reconstruction is acceptable.

        disp(' ');
        result = ['  Results: lambda=' num2str(lambda) ', mean(chi2)=' num2str(chi2)];
        disp(result);
        disp(['  WARNING: second reconstruction not successful for shot ' int2str(shot) ' !']);
        disp(' ');
        errflag=1;

      else  
        
        disp('  ');
        result = ['shot ',int2str(shot),' OK. (lambda = ', num2str(lambda), ...
                       ', chi2 = ', num2str(chi2),')'];
        good = 1;
        disp(result);
        disp(' ');
        
      end
  
    end
  
  end

  X = reshape(Xinv, ny*nx, length(tvec));

  rm = xmesh / 100;      % Conversion to [m]  
  zm = ymesh / 100;      % Conversion to [m] 

  if radiation       % Optional calculation of the radiation fractions issued inside and
                     % outside of the LCFS surface.

    disp(['Calculation of the LCFS fractions of the radiation started.']);
   
 	[Ptot, Pin, Pout, Pabove, Pbelow, tvec] = ...
			fabpower(X, shot, tvec, xmesh, ymesh, 1);
      
  end

  mdsclose;

  if saving          % Optinal saving of variables on harddisk.
      
    savefile=['FABCAT_',int2str(shot)];
   
    try

      eval(['save ',[savepath,savefile],' shot Xinv tvec FastPtot FastPbelow FastPxpoint good ']);
      disp(['The following file has been created: ', savepath savefile]); 
      
    end   
    
  end

  % SEVENTH STEP : Fast calculation of the radiated power

  for k = 1 : nx

    Pinv(:,k,:) = Xinv(:,k,:)*xmesh(k); 
     
                          % Integrand for later integration
       
  end
   
  fact=2*pi*dx*dy/1e6;    % Integration factor (over toroidal ring).
  
  P = reshape(Pinv,nx*ny,length(tvec));
  
                          % Pixel power matrix
                          
  FastPtot = sum(P)*fact; % Sum over all pixels
  
  if ~isempty(K)
    FastPbelow = (sum(P(1:(K-2),:)) + sum(P(29:(K+26),:))) * fact;
    
                          % Power radiated from the divertor region.
                          
    FastPxpoint = (sum(P(K-1:K+1,:)) + sum(P(K+27:K+29,:))+P(K+56,:)) * fact;
    
                          % Power radiated from the expanded X-point.
                          
  else 
  
    FastPbelow = NaN;
    FastPxpoint = NaN;
    
  end
     
  if MDS                  % Optional saving to the MDS tree.
  
    if xpoint             % Comments for MDS node 'COMMENT'
    
      mtxt = ['FABCAT with X-point at pixel ', num2str(K)];
    
    else     
  
      mtxt = ['FABCAT without X-point at pixel ', num2str(K)];
    
    end
   
    if isempty(K)         % No X-point found.

      mtxt = ['FABCAT, X-point determination failed'];
       
    end

    fabstore(shot, rm, zm, tvec, X, lambda, runsigma, chim, points, setdiff(1:64,inumdet), mtxt, result, FastPtot, NaN, NaN, FastPxpoint, FastPbelow);

  end
  
end

time_end = cputime - debut;
disp(['CPU runtime: ' num2str(time_end),' s.']);
disp('  ');
