function [Ptot, Pin, Pout, Pabove, Pbelow, tvec] = ...
            fabpower(X, shot, tvec, rm, zm, iplot);
        
% function [Ptot, Pin, Pout, Pabove, Pbelow, tvec] = ...
%            fabpower(X, shot, tvec, rm, zm, iplot);
%									;
% Calculates the cell- and grid-points of a selected
% mesh (rm, zm in [cm]) for tomographic inverson of
% the bolometric data lying inside/outside the LCFS.
% Also determines the cutpoints of the lcfs with the tomogrid
% and calculates Ptot, Pin, Pout, and Pdiv from the power
% radiated in-/outside the lcfs from each cell
%
% This routine was taken from the 'btomo.m' package
%
% History:        
%
%  Arno Refke, CRPP EPFL, 18 of September, 1996 (first release)
%  Iwan Jerjen, CRPP EPFL, 26 of August, 1999 (minor modifications)
%
% Final release version 1.4 (2002/02/07)

global ha_figrad	%handle of figure

if nargin < 6
   iplot = 0;
   else iplot=iplot;
end

% Normalize

calf=max(X(:));	% scalar
X=X/calf;

% Create vector Pm to every mesh-point

nx = length(rm);		% number of x-cells
ny = length(zm);		% number of y cells
dx = (rm(2)-rm(1));		% cell size in x
dy = (zm(2)-zm(1));		% cell size in y
xmax = rm(nx)+dx/2;		% maximum x mesh point
xmin = rm(1)-dx/2;		% minimum x mesh point
ymax = zm(ny)+dy/2;		% maximum y mesh point
ymin = ymax-ny*dy;		% minimum y mesh point
Acell = dx*dy;			% cell area;

xgrid = [xmin:dx:xmax+0.001];		% vector of x grid points
ygrid = fliplr([ymin:dy:ymax+0.001]);	% vector of y grid points

mx = length(xgrid);		% number of x meshgrid points
my = length(ygrid);		% number of y meshgrid-points
km = mx*my;			% total number of mesh points

Pm=zeros(km,3);
for j=1:my
Pm((mx*(j-1)+1):(j*mx),1) = xgrid';
end
for j=1:mx
Pm(j:mx:(mx*(my-1)+j),2) = ygrid';
end

% Create vector Pc to every cell center-point
% Only xcell and kc are used

xcell = rm;			% vector of x cell center points
kc = nx*ny;			% number of cell_points

if length(calf) == 1, calf = calf*ones(length(tvec),1); end;

% Loop over all times specified in the shot

for l=1:length(tvec);

% Vector L to every point of the LCFS

[con_xt,con_yt,c_xt,c_yt,times]=fablcfs(shot,tvec(l));

I = find(~isnan(con_xt) & ~isnan(con_yt));
con_xt = con_xt(I);
con_yt = con_yt(I);

lcfs_x = con_xt;		% vector of lcfs x-points
lcfs_y = con_yt;		% vector of lcfs y-points
y_Xpt  = min(lcfs_y);	% y - coordinate of the X-point;
y_Tpt  = max(lcfs_y);	% y - coordinate of the top-point from LCFS;
i = length(lcfs_x);		% number of lcfs-points
L=zeros(i,2);
L(:,1) = lcfs_x;
L(:,2) = lcfs_y;

% Matrix Rm with vectors from each mesh point to each lcfs-point

Rm = zeros(i,2*km);
for j=1:2:(2*km-1)
Rm(:,j)   = L(:,1)-Pm((j+1)/2,1);
Rm(:,j+1) = L(:,2)-Pm((j+1)/2,2);
end

% Determine for each mesh point the angle phi_m from the polarcoordinates of Rm

phi_m = zeros(i,km);
phi_m = atan2(Rm(:,2:2:2*km),Rm(:,1:2:(2*km)-1));
phi_mn =phi_m;	% phi_mn element [-pi,pi]

for j=1:i,
    for k=1:km,
	if phi_m(j,k)<0,
	   phi_m(j,k)=phi_m(j,k)+2*pi;	% phi_m element [0,2*pi]
	end
    end
end

dif_phi_m = max(phi_m)-min(phi_m);
dif_phi_mn = max(phi_mn)-min(phi_mn);
concavefactor=1.07;

for j=1:km
	if  dif_phi_m(j) > concavefactor*pi & dif_phi_mn(j) > concavefactor*pi
		Pm(j,3) = 1;
	else
		Pm(j,3) = 0;
	
	end	
end

[Mout] = find(Pm(:,3) == 0);
[Min]  = find(Pm(:,3) == 1);

% Define different branches of the lcfs for use of interpolation

 [XMIN,N1]=min(con_xt);
 [XMAX,N2]=max(con_xt);
 [YMIN,N3]=min(con_yt);
 [YMAX,N4]=max(con_yt);
 
 lcfs_xu = fliplr(con_xt(N2:N1));     
 lcfs_yu = fliplr(con_yt(N2:N1));   
 lcfs_xo = [con_xt(N1:length(con_xt));con_xt(2:N2)];  
 lcfs_yo = [con_yt(N1:length(con_yt));con_yt(2:N2)];    

 if N3>N4, 
	Nl = [N3:length(con_yt) 1:N4];
	Nr = [N4:N3];
	lcfs_xl = con_xt(Nl);     
	lcfs_yl = con_yt(Nl);     
	lcfs_xr = con_xt(Nr);
	lcfs_yr = con_yt(Nr);
 else	lcfs_xl = con_xt(N3:N4);     
 	lcfs_yl = con_yt(N3:N4);   
	lcfs_xr = fliplr([con_xt(N4:length(con_xt));con_xt(2:N3)]);  
	lcfs_yr = fliplr([con_yt(N4:length(con_yt));con_yt(2:N3)]);
 end   
    
% Make sure that each branch is monotonic

lcfs_xo_dif = diff(lcfs_xo);
[Mxo] = find(lcfs_xo_dif<0);
if ~isempty(Mxo)
   [lcfs_xo,I] = sort(lcfs_xo);
   for m=1:length(lcfs_xo),
	lcfs_yo(m) = lcfs_yo(I(m));	
   end
end
lcfs_xo_dif = diff(lcfs_xo);
[Nxo] = find(lcfs_xo_dif==0);
if ~isempty(Nxo)
 Nxo_dif = diff(Nxo);
 [nxo] = find(Nxo_dif==1);
 if length(Nxo)>0,
   if ~isempty(nxo)
	for m=1:length(nxo),
	lcfs_xo(Nxo(nxo(m))) = (lcfs_xo(Nxo(nxo(m)))+lcfs_xo(Nxo(nxo(m))-1))/1.99999;
	end
   end
   for m=1:length(Nxo),
	if Nxo(m) < length(lcfs_xo)-1,
	lcfs_xo(Nxo(m)+1) = (lcfs_xo(Nxo(m)+1)+lcfs_xo(Nxo(m)+2))/1.99999;
	else lcfs_xo(Nxo(m)) = (lcfs_xo(Nxo(m))+lcfs_xo(Nxo(m)-1))/1.99999;
	end
   end
 end
end
lcfs_xu_dif = diff(lcfs_xu);
[Mxu] = find(lcfs_xu_dif<0);
if ~isempty(Mxu)
   [lcfs_xu,I] = sort(lcfs_xu);
   for m=1:length(lcfs_xu),
	lcfs_yu(m) = lcfs_yu(I(m));	
   end
end
lcfs_xu_dif = diff(lcfs_xu);
[Nxu] = find(lcfs_xu_dif==0);
if ~isempty(Nxu)
 Nxu_dif = diff(Nxu);
 [nxu] = find(Nxu_dif==1);
 if length(Nxu)>0,
   if ~isempty(nxu)
	for m=1:length(nxu),
	lcfs_xu(Nxu(nxu(m))) = (lcfs_xu(Nxu(nxu(m)))+lcfs_xu(Nxu(nxu(m))-1))/1.99999;
	end
   end	
   for m=1:length(Nxu),
	if Nxu(m) < length(lcfs_xu)-1,
	    lcfs_xu(Nxu(m)+1) = (lcfs_xu(Nxu(m)+1)+lcfs_xu(Nxu(m)+2))/1.99999;
	else lcfs_xu(Nxu(m)) = (lcfs_xu(Nxu(m))+lcfs_xu(Nxu(m)-1))/1.99999;
	end
   end
 end
end
lcfs_yl_dif = diff(lcfs_yl);
[Myl] = find(lcfs_yl_dif<0);
if ~isempty(Myl)
   [lcfs_yl,I] = sort(lcfs_yl);
   for m=1:length(lcfs_yl),
	lcfs_xl(m) = lcfs_xl(I(m));	
   end
end
lcfs_yl_dif = diff(lcfs_yl);
[Nyl] = find(lcfs_yl_dif==0);
if ~isempty(Nyl)
 Nyl_dif = diff(Nyl);
 [nyl] = find(Nyl_dif==1);
 if length(Nyl)>0,
   if ~isempty(nyl)
	for m=1:length(nyl),
	lcfs_yl(Nyl(nyl(m))) = (lcfs_yl(Nyl(nyl(m)))+lcfs_yl(Nyl(nyl(m))-1))/1.99999;
	end
   end	
   for m=1:length(Nyl),
	if Nyl(m) < length(lcfs_yl)-1,
	lcfs_yl(Nyl(m)+1) = (lcfs_yl(Nyl(m)+1)+lcfs_yl(Nyl(m)+2))/1.99999;
	else lcfs_yl(Nyl(m)) = (lcfs_yl(Nyl(m))+lcfs_yl(Nyl(m)-1))/1.99999;
	end
   end
 end
end
lcfs_yr_dif = diff(lcfs_yr);
[Myr] = find(lcfs_yr_dif<0);
if ~isempty(Myr)
   [lcfs_yr,I] = sort(lcfs_yr);
   for m=1:length(lcfs_yr),
	lcfs_xr(m) = lcfs_xr(I(m));	
   end
end
lcfs_yr_dif = diff(lcfs_yr);
[Nyr] = find(lcfs_yr_dif==0);
if ~isempty(Nyr)
 Nyr_dif = diff(Nyr);
 [nyr] = find(Nyr_dif==1);
 if length(Nyr)>0,
   if ~isempty(nyr)
	for m=1:length(nyr),
	lcfs_yr(Nyr(nyr(m))) = (lcfs_yr(Nyr(nyr(m)))+lcfs_yr(Nyr(nyr(m))-1))/1.99999;
	end
   end	
   for m=1:length(Nyr),
	if Nyr(m) < length(lcfs_yr)-1,
	lcfs_yr(Nyr(m)+1) = (lcfs_yr(Nyr(m)+1)+lcfs_yr(Nyr(m)+2))/1.99999;
	else lcfs_yr(Nyr(m)) = (lcfs_yr(Nyr(m))+lcfs_yr(Nyr(m)-1))/1.99999;
	end
   end
 end
end

% Calculation of Power radiated inside/outside lcfs

pin(l) = 0;
pout(l) = 0;
pdiv(l) = 0;
ptop(l) = 0;
P_in(l) = 0;
P_out(l) = 0;
P_div(l) = 0;
P_top(l) = 0;

cell_inout = 2*ones(kc);
emiss = reshape(X(:,l),length(zm),length(rm));
emiss = flipud(emiss);

for j=1:ny
    for k=1:nx
	if Pm((j-1)*mx+k,3)==1 & Pm((j-1)*mx+k+1,3)==1 & Pm(j*mx+k,3)==1 & Pm(j*mx+k+1,3)==1,
	   cell_inout((j-1)*nx+k) = 1;
	  
	   pin(l) = pin(l) + emiss(j,k)*xcell(k);   
    elseif Pm((j-1)*mx+k,3)==0 & Pm((j-1)*mx+k+1,3)==0 & Pm(j*mx+k,3)==0 & Pm(j*mx+k+1,3)==0,
	   cell_inout((j-1)*nx+k) = 0;
	   pout(l) = pout(l) + emiss(j,k)*xcell(k);
 	   
% Determines the cutlines of each cell with the lcfs

elseif Pm((j-1)*mx+k,3)==0 & Pm((j-1)*mx+k+1,3)==0 & Pm(j*mx+k,3)==1 & Pm(j*mx+k+1,3)==1,
 
 x_cut(1) = xgrid(k);
 y_cut(1) = interp1(lcfs_xo,lcfs_yo,x_cut(1));
 x_cut(2) = xgrid(k+1);
 y_cut(2) = interp1(lcfs_xo,lcfs_yo,x_cut(2));
 Ain  = ((min(y_cut)-ygrid(j+1))*dx + abs(diff(y_cut))*dx/2) / Acell;
 P_in(l)  = P_in(l) + emiss(j,k)*xcell(k)*Ain;
 P_out(l) = P_out(l) + emiss(j,k)*xcell(k)*(1-Ain);


elseif Pm((j-1)*mx+k,3)==1 & Pm((j-1)*mx+k+1,3)==1 & Pm(j*mx+k,3)==0 & Pm(j*mx+k+1,3)==0,
 x_cut(1) = xgrid(k+1);
 y_cut(1) = interp1(lcfs_xu,lcfs_yu,x_cut(1));
 x_cut(2) = xgrid(k);
 y_cut(2) = interp1(lcfs_xu,lcfs_yu,x_cut(2));
 Aout = ((min(y_cut)-ygrid(j+1))*dx + abs(diff(y_cut))*dx/2) / Acell;
 P_in(l) = P_in(l) + emiss(j,k)*xcell(k)*(1-Aout);
 P_out(l) = P_out(l) + emiss(j,k)*xcell(k)*Aout;

elseif Pm((j-1)*mx+k,3)==0 & Pm((j-1)*mx+k+1,3)==1 & Pm(j*mx+k,3)==0 & Pm(j*mx+k+1,3)==1,
 y_cut(1) = ygrid(j+1);
 x_cut(1) = interp1(lcfs_yl,lcfs_xl,y_cut(1));
 y_cut(2) = ygrid(j);
 x_cut(2) = interp1(lcfs_yl,lcfs_xl,y_cut(2));
 Ain  = ((min(x_cut)-xgrid(k))*dy + abs(diff(x_cut))*dy/2) / Acell;
 P_in(l)  = P_in(l) + emiss(j,k)*xcell(k)*Ain;
 P_out(l) = P_out(l) + emiss(j,k)*xcell(k)*(1-Ain);

elseif Pm((j-1)*mx+k,3)==1 & Pm((j-1)*mx+k+1,3)==0 & Pm(j*mx+k,3)==1 & Pm(j*mx+k+1,3)==0,
 y_cut(1) = ygrid(j);
 x_cut(1) = interp1(lcfs_yr,lcfs_xr,y_cut(1));
 y_cut(2) = ygrid(j+1);
 x_cut(2) = interp1(lcfs_yr,lcfs_xr,y_cut(2));
 Aout = ((min(x_cut)-xgrid(k))*dy + abs(diff(x_cut))*dy/2) / Acell;
 P_in(l)  = P_in(l) + emiss(j,k)*xcell(k)*(1-Aout);
 P_out(l) = P_out(l) + emiss(j,k)*xcell(k)*Aout;

elseif Pm((j-1)*mx+k,3)==0 & Pm((j-1)*mx+k+1,3)==0 & Pm(j*mx+k,3)==0 & Pm(j*mx+k+1,3)==1,
 y_cut(1) = ygrid(j+1);
 x_cut(1) = interp1(lcfs_yl,lcfs_xl,y_cut(1));
 x_cut(2) = xgrid(k+1);
 y_cut(2) = interp1(lcfs_xo,lcfs_yo,x_cut(2));
 Ain  = abs(diff(x_cut))*abs(diff(y_cut))/2 / Acell;
 P_in(l)  = P_in(l) + emiss(j,k)*xcell(k)*Ain;
 P_out(l) = P_out(l) + emiss(j,k)*xcell(k)*(1-Ain);

elseif Pm((j-1)*mx+k,3)==0 & Pm((j-1)*mx+k+1,3)==0 & Pm(j*mx+k,3)==1 & Pm(j*mx+k+1,3)==0,
 y_cut(2) = ygrid(j+1);
 x_cut(2) = interp1(lcfs_yr,lcfs_xr,y_cut(2));
 x_cut(1) = xgrid(k);
 y_cut(1) = interp1(lcfs_xo,lcfs_yo,x_cut(1));
 Ain  = abs(diff(x_cut))*abs(diff(y_cut))/2 / Acell;
 P_in(l)  = P_in(l) + emiss(j,k)*xcell(k)*Ain;
 P_out(l) = P_out(l) + emiss(j,k)*xcell(k)*(1-Ain);

elseif Pm((j-1)*mx+k,3)==1 & Pm((j-1)*mx+k+1,3)==0 & Pm(j*mx+k,3)==0 & Pm(j*mx+k+1,3)==0,
 y_cut(1) = ygrid(j);
 x_cut(1) = interp1(lcfs_yr,lcfs_xr,y_cut(1));
 x_cut(2) = xgrid(k);
 y_cut(2) = interp1(lcfs_xu,lcfs_yu,x_cut(2));
 Ain  = abs(diff(x_cut))*abs(diff(y_cut))/2 / Acell;
 P_in(l)  = P_in(l) + emiss(j,k)*xcell(k)*Ain;
 P_out(l) = P_out(l) + emiss(j,k)*xcell(k)*(1-Ain);

elseif Pm((j-1)*mx+k,3)==0 & Pm((j-1)*mx+k+1,3)==1 & Pm(j*mx+k,3)==0 & Pm(j*mx+k+1,3)==0,
 y_cut(2) = ygrid(j);
 x_cut(2) = interp1(lcfs_yl,lcfs_xl,y_cut(2));
 x_cut(1) = xgrid(k+1);
 y_cut(1) = interp1(lcfs_xu,lcfs_yu,x_cut(1));
 Ain  = abs(diff(x_cut))*abs(diff(y_cut))/2 / Acell;
 P_in(l)  = P_in(l) + emiss(j,k)*xcell(k)*Ain;
 P_out(l) = P_out (l)+ emiss(j,k)*xcell(k)*(1-Ain);

elseif Pm((j-1)*mx+k,3)==0 & Pm((j-1)*mx+k+1,3)==1 & Pm(j*mx+k,3)==1 & Pm(j*mx+k+1,3)==1,
 x_cut(1) = xgrid(k);
 y_cut(1) = interp1(lcfs_xo,lcfs_yo,x_cut(1));
 y_cut(2) = ygrid(j);
 x_cut(2) = interp1(lcfs_yl,lcfs_xl,y_cut(2));
 Aout = abs(diff(x_cut))*abs(diff(y_cut))/2 / Acell;
 P_in(l)  = P_in(l) + emiss(j,k)*xcell(k)*(1-Aout);
 P_out(l) = P_out(l)+ emiss(j,k)*xcell(k)*Aout;

elseif Pm((j-1)*mx+k,3)==1 & Pm((j-1)*mx+k+1,3)==0 & Pm(j*mx+k,3)==1 & Pm(j*mx+k+1,3)==1,
 x_cut(2) = xgrid(k+1);
 y_cut(2) = interp1(lcfs_xo,lcfs_yo,x_cut(2));
 y_cut(1) = ygrid(j);
 x_cut(1) = interp1(lcfs_yr,lcfs_xr,y_cut(1));
 Aout = abs(diff(x_cut))*abs(diff(y_cut))/2 / Acell;
 P_in(l) = P_in(l) + emiss(j,k)*xcell(k)*(1-Aout);
 P_out(l) = P_out(l) + emiss(j,k)*xcell(k)*Aout;

elseif Pm((j-1)*mx+k,3)==1 & Pm((j-1)*mx+k+1,3)==1 & Pm(j*mx+k,3)==1 & Pm(j*mx+k+1,3)==0,
 x_cut(1) = xgrid(k+1);
 y_cut(1) = interp1(lcfs_xu,lcfs_yu,x_cut(1));
 y_cut(2) = ygrid(j+1);
 x_cut(2) = interp1(lcfs_yr,lcfs_xr,y_cut(2));
 Aout = abs(diff(x_cut))*abs(diff(y_cut))/2 / Acell;
 P_in(l)  = P_in(l) + emiss(j,k)*xcell(k)*(1-Aout);
 P_out(l) = P_out(l) + emiss(j,k)*xcell(k)*Aout;

elseif Pm((j-1)*mx+k,3)==1 & Pm((j-1)*mx+k+1,3)==1 & Pm(j*mx+k,3)==0 & Pm(j*mx+k+1,3)==1,
 x_cut(2) = xgrid(k);
 y_cut(2) = interp1(lcfs_xu,lcfs_yu,x_cut(2));
 y_cut(1) = ygrid(j+1);
 x_cut(1) = interp1(lcfs_yl,lcfs_xl,y_cut(1));
 Aout = abs(diff(x_cut))*abs(diff(y_cut))/2 / Acell;
 P_in(l)  = P_in(l) + emiss(j,k)*xcell(k)*(1-Aout);
 P_out(l) = P_out(l) + emiss(j,k)*xcell(k)*Aout;
	
	end
    end 
end 

% Calculation of power radiated in the divertor (below the X-point)

for j=1:ny
    for k=1:nx

	if  Pm((j-1)*mx+k,2)<y_Xpt & Pm((j-1)*mx+k+1,2)<y_Xpt & Pm(j*mx+k,2)<y_Xpt & Pm(j*mx+k+1,2)<y_Xpt,
	    pdiv(l) = pdiv(l) + emiss(j,k)*xcell(k);

    elseif  Pm((j-1)*mx+k,2)>y_Xpt & Pm((j-1)*mx+k+1,2)>y_Xpt & Pm(j*mx+k,2)<y_Xpt & Pm(j*mx+k+1,2)<y_Xpt,
	    x_cut(1) = xgrid(k);
	    x_cut(2) = xgrid(k+1);
	    Adiv  = diff(x_cut)*(y_Xpt - ygrid(j+1))/Acell;
	    P_div(l) = P_div(l) + emiss(j,k)*xcell(k)*Adiv;	  
	end 	   
    end
end

% Calculation of power radiated above the LCFS
for j=1:ny
    for k=1:nx

	if  Pm((j-1)*mx+k,2)>y_Tpt & Pm((j-1)*mx+k+1,2)>y_Tpt & Pm(j*mx+k,2)>y_Tpt & Pm(j*mx+k+1,2)>y_Tpt,
	    ptop(l) = ptop(l) + emiss(j,k)*xcell(k);

    elseif  Pm((j-1)*mx+k,2)>y_Tpt & Pm((j-1)*mx+k+1,2)>y_Tpt & Pm(j*mx+k,2)<y_Tpt & Pm(j*mx+k+1,2)<y_Tpt,
	    x_cut(1) = xgrid(k);
	    x_cut(2) = xgrid(k+1);
	    Atop  = diff(x_cut)*(ygrid(j)-y_Tpt)/Acell;
	    P_top(l) = P_top(l) + emiss(j,k)*xcell(k)*Atop;	  
	end 	   
    end
end

[IN]  = find(cell_inout == 1); 
[OUT] = find(cell_inout == 0); 

% Power radiated inside / outside LCFS from cells

P_in(l)  = calf(l)*P_in(l)*2*pi*dx*dy;	% partially inside  lcfs
P_out(l) = calf(l)*P_out(l)*2*pi*dx*dy;	% partially outside lcfs
P_div(l) = calf(l)*P_div(l)*2*pi*dx*dy;	% partially below   lcfs
P_top(l) = calf(l)*P_top(l)*2*pi*dx*dy;	% partially above   lcfs
pin(l)   = calf(l)*pin(l)*2*pi*dx*dy;	% totally inside  lcfs
pout(l)  = calf(l)*pout(l)*2*pi*dx*dy;	% totally outside lcfs
pdiv(l)  = calf(l)*pdiv(l)*2*pi*dx*dy;	% totally below   lcfs
ptop(l)  = calf(l)*ptop(l)*2*pi*dx*dy;	% totally above   lcfs
Pin(l)   = (pin(l) + P_in(l))*1e-6;	    % inside  lcfs rad. power
Pout(l)  = (pout(l)+ P_out(l))*1e-6;	% outside lcfs rad. power
Pdiv(l)  = (pdiv(l)+ P_div(l))*1e-6;	% below   lcfs rad. power
Ptop(l)  = (ptop(l)+ P_top(l))*1e-6;	% above   lcfs rad. power
Ptot(l)  = (Pin(l) + Pout(l));	        % totally radiated power
end

Pabove = Ptop;
Pbelow = Pdiv;

% Plots radiated power fractions (Ptot, Pin, Pout and Pdiv)
% Calculated from bolometric inversion routine

if iplot
  ha_figrad=figure('Name','btomorun radiation fractions window', ...
	'NumberTitle','off');
  clf	
  hmenu=uimenu('Parent',ha_figrad, ...
  	  'Label','E&XIT', ...
	  'Callback','close(gcf)');	
  
  plot(tvec,Ptot/1e+3,'b')
  hold on
  plot(tvec,Pin/1e+3,'r')
  plot(tvec,Pout/1e+3,'g')
  plot(tvec,Pdiv/1e+3,'-.m')
  plot(tvec,Ptop/1e+3,'--c')
  title(['# ',int2str(shot),' bolometric inversion']);
  xlabel('time  [s]');
  ylabel('Prad  [kW]');
  legend('Ptot','Pin','Pout','Pbelow','Pabove',0);
end