function [gres, lamout, chi2out, actt] = ...
         fabrec(Y, dY, xmesh, ymesh, tvec, actt, actdet, Tb, ...
                  lambdai, ifishmax, iimax, i_bord, limit);

% function [gres, lamout, chi2out, actt] = ...
%         fabrec(Y, dY, xmesh, ymesh, tvec, actt, actdet, Tb, ...
%                  lambdai, ifishmax, iimax, i_bord, limit);
%
% Tomographic reconstruction of the emissivity for the given
% signals and geometry. The reconstruction is based on minimizing
% the Fisher information and uses a single reconstruction matrix
% which is averaged over all timeframes.
%
% The routine does a post-reconstruction test for bad channels and exclude
% them from the reconstruction.
%
% Outputs:  gres    : the emissivity array, size (ny x nx x length(tvec))
%           lamout  : the final smoothing factor LAMBDA
%           chi2out : The final chi^2, length(chi2out) = lenght(tsteps)
%           actt    : The array of enabled bolometer channels
%
% Please not that the average of chi2 = mean(chi2out) is targeted to 1,
% representing then the solution with maximum variance (dY).
%
% History:        
%
%  Jan Mlynarr, CRPP EPFL, May 2001 (first release)
%  dito, January 2002 (crosscheck of good channels)
%  
% Final release version 1.4 (2002/02/07)

fprintf('FABCAT inversion routine. Release 1.4 (2002/02/07)\n');

while 1 % Repetitive execution until no more weird channels are discovered
        % by the post-reconstruction check (see at the end of this routine)

lambda=lambdai;
To=Tb(actt,:);               % Restrictions to enabled bolometers
sigmao=dY(actt);
YY=Y(actdet,:);

tsteps=length(tvec);
[nl,npix]=size(To);

if i_bord                    % no border condition

  chi2tar=(nl+1)*tsteps;     % chi2 per a sample should be 1
                             % (nl+1) one in plus - see the zero border below  
else

  chi2tar=nl*tsteps;         % chi2 per a sample should be 1

end

dx=xmesh(2)-xmesh(1);
dy=ymesh(2)-ymesh(1);
nx=length(xmesh);
ny=length(ymesh);
 
% START of the fast interim of of the cattcv package 'makem_1.m'
% only used when the no border condition is enabled.

epsilon=sqrt(eps);

% Border pixel determination

iborder=[1:ny,ny+1:ny:((ny*(nx-2))+1),(2:(nx-1))*ny,(1+(nx-1)*ny):nx*ny];
nborder=length(iborder);

Ymax=max(YY);
D=max(YY./repmat(Ymax,nl,1),epsilon*ones(nl,tsteps));

% Default model for the tought-for emissivity distribution.

Tbar=sum(To(:))/(nl+npix);
Ep=D./repmat(sum(To')',1,tsteps);
nn=find(sum(To)~=0); 
Eca=(Ep'*To(:,nn))./repmat(sum(To(:,nn)),tsteps,1);
 
% Totally flat model with zero border.

I0=sum(Eca');
Emean=I0./(npix-nborder);
g_model=repmat(Emean,npix,1);
g_model(iborder,:)=1e-9*ones(length(iborder),tsteps);
g_model=g_model.*repmat(Ymax,npix,1);

% END of fast interim of "makem_1.m"
  
tiny=1e-4;
dfadd=min(sigmao)/100;	% Relative error of the lines of sight
	 
i_pocs=1;
rgmin=1e-2;

diam=eye(npix);				        % diagonal
diar=diag(ones(1,npix-ny),ny);		% to reference right nearest neighbors
dial=diag(ones(1,npix-ny),-ny);		% reference left nearest neighbors
diao=diag(ones(1,npix-1),1);		% reference upper nearest neighbors
diau=diag(ones(1,npix-1),-1);		% reference lower nearest neighbors

% Indices for borders and coins

i_bl=2:ny-1;			            % left border
i_br=(2+(nx-1)*ny):(npix-1);	    % right border
i_bo=(2:(nx-1))*ny;		            % upper border
i_bu=1+(1:nx-2)*ny;		            % lower border
i_ul=1;				                % lower left corner
i_ol=ny;			                % upper left corner
i_or=npix;			                % upper right corner
i_ur=1+(nx-1)*ny;          		    % lower right corner
i_oben=[i_ol,i_bo,i_or];
i_rechts=[i_ur,i_br,i_or];

% Evaluation of the gradients

Bx=-diam+diar;                      % finite-difference matrix representations
By=-diam+diao;                      % of the partial derivates.

By(i_oben,:)=diam(i_oben,:)+diau(i_oben,:);
Bx(i_rechts,:)=diam(i_rechts,:)+dial(i_rechts,:);

Bx=Bx/dx;
By=By/dy;

% Normalization and division by the variance

fo=Y(actdet,:);
noemiss=[];                         % border fixed to zero: 

if i_bord                           % border pixels emissivity set to zero.

  noemiss=find(g_model<=tiny*max(max(g_model)));	
  Tadd=zeros(1,npix);
  inx=find(noemiss<npix);
  Tadd(noemiss(inx))=ones(size(inx));
  fadd=zeros(1,tsteps);
  T=[To;Tadd];
  f=[fo;fadd];   
  sigma=[sigmao;dfadd];
  
else
     
  f = fo;
  T = To;
  sigma = sigmao;
  
end

[nl,npix]=size(T);	% size of T changes for the fixed border

fmax = max(f);
f = f./repmat(fmax,size(f,1),1);     % Normalization  
 
S1=diag(1./sigma);                   % Division by the variance
TT=zeros(npix);
Ts=S1*T;
TT=Ts'*Ts;   

f(find(f<0)) = zeros(size(find(f<0)));	% no negatives emissions:
fs = f./repmat(sigma,1,tsteps);

lam=zeros(ifishmax,iimax);
chi2=lam;
chi2det=zeros(ifishmax,iimax,tsteps);
chi2b=zeros(ifishmax,tsteps);

% *** Mainloop to minimize the Fisher information

for i=1:ifishmax

  if i==1 % First loop
		
    w = ones(npix,1);                  % unity weight matrix
                                       % The first iteration corresponds
                                       % to linear regularization
  	H = Bx'*Bx+By'*By;	               % The H-matrix
    coeflam=trace(TT)/trace(H);        % inital LAMBDA    
    lam(1,1)=lambda*coeflam;         
    A=(TT+lam(1,1)*H)';
	Tpsinv=A\Ts';                      % The M-matrix
	g=Tpsinv*fs;	                   % The emissivity
 
    neg=find(g<0);
    g(neg)=zeros(size(neg));           % Negative emissivity is set to zero
    g(noemiss)=zeros(size(noemiss));   % the same with noemission pixels

  else
		 
	gdummy=sum(g,2)/max(sum(g,2));
	petit=find(gdummy<rgmin);
	gdummy(petit)=rgmin*ones(size(petit));
	w=1./gdummy;                       % The weight matrix W(n), n>1
    Ax=diag(w)*Bx;
    Ay=diag(w)*By;
	H=Bx'*Ax+By'*Ay;	               % H(n), n>1
       
    lam(i,1)=lambdab(i-1);
    A=(TT+lam(i,1)*H)';     
	Tpsinv=A\Ts';                      % M(n), n>1
    g=Tpsinv*fs;	
 
    neg=find(g<0);
    g(neg)=zeros(size(neg)); 
    g(noemiss)=zeros(size(noemiss)); 
    
  end       
      
  lam(i,2)=lam(i,1)/2;
  fc=Ts*g;                             % Reconstructed chord brightness
  dummy=fc-fs;                         
  chi2det(i,1,:)=sum(dummy.*dummy)';   % (ifishmax * iimax * tsteps) matrix
  chi2(i,1)=sum(chi2det(i,1,:));

% *** Subloop to find LAMBDA which delivers chi^2 -> target chi^2  

  for ii=2:iimax 
   
    A=(TT+lam(i,ii)*H)';
    Tpsinv=A\Ts';
    g=Tpsinv*fs; 
	neg=find(g<0);
	g(neg)=zeros(size(neg)); 
	g(noemiss)=zeros(size(noemiss));          
    fc=Ts*g;
    dummy=fc-fs;
	chi2det(i,ii,:)=sum(dummy.*dummy)';
    chi2(i,ii)=sum(chi2det(i,ii,:));

    if ii==iimax break; end           % If last subloop iteration reached

    deriv=(lam(i,ii)-lam(i,ii-1))/(chi2(i,ii)-chi2(i,ii-1));
    coef=lam(i,ii)-deriv*chi2(i,ii);
	lam(i,ii+1)=deriv*chi2tar+coef;   % The new lambda
	
    if lam(i,ii+1)<=0 lam(i,ii+1)=lam(i,ii)/2; end

	ii=ii+1;		
    
  end % of the subloop
  
  [noth,stepno]=min(abs(chi2(i,:)-chi2tar*ones(size(chi2(i,:)))));
                                      
                                      % find lambda which makes chi2 closest to chi2tar
	
  chi2b(i,:)=chi2det(i,stepno,:);
  lambdab(i)=lam(i,stepno);           % Best LAMBDA so far
  A=(TT+lambdab(i)*H)';
  Tpsinv=A\Ts';                       % Best M-matrix so far
  g=Tpsinv*fs;                        % Best emissivity reconstruction so far
  neg=find(g<0);
  g(neg)=zeros(size(neg)); 
  g(noemiss)=zeros(size(noemiss));
  
end % of the main loop
       
lambda=lam/coeflam;                   % (ifishmax * iimax) matrix, describes the iterative
                                      % evolution of smoothing
lamout=lambda(ifishmax,iimax);        % Final LAMBDA
chi2out=chi2b(ifishmax,:)/nl;         % Final chi^2 vector

gres=reshape(g.*repmat(fmax,npix,1),ny,nx,length(tvec));

                                      % Final emissivity

% Post-reconstruction bad channel detection

mdum=mean(dummy,2);                   % Mean chi
stdd=std(mdum);                       % standard deviation of mean chi
goodch=find(abs(mdum)<limit*stdd);    % LIMIT determines the confidential channels

if length(goodch)==length(mdum) break; end

delch=find(abs(mdum)>=limit*stdd);
fprintf('\n');
fprintf('  Channels deleted by the chi2 test : ');
fprintf('%1.0f,', actt(delch'));
fprintf('\n\n');

actt=actt(goodch);
actdet=actdet(goodch);
  
end
