function [framedata,frametbase,errflag,errstr,goodchans] = ...
     fabsig(btshotnum,f_cam,delchanuser,smoothmode,tmin,tmax,points,chantest)

% function [framedata,frametbase,errflag,errstr,goodchans] = ...
%     fabsig(btshotnum,f_cam,delchanuser,smoothmode,tmin,tmax,points,chantest)
%
% Gets the bolometry signals for the tomography analysis, checks for bad 
% bolometers and does data processing (calibration). This is the fast version.
%
% This function has been rewritten to satisfy the needs of bolometry.
% It is based on btomodata.m written by Jan Mlynar in 1999.
%
% If tmin is a number, it takes all time samples within tmin and tmax.
% Otherwise it loads data only at LIUQE times
%
% History:
%
%  Christian Schlatter, TPIV CRPP EPFL, February 2002
%  major parts stolen from the btomo package
%
% Release 1.4 (2002/02/07)

if nargin<5 tmin = NaN; end
if nargin<7 points = NaN; end

fprintf('FABCAT signal processing routine. Release 1.4 (2002/02/07)\n');

fprintf('Loading of the bolometry signals.\n\n');

errflag = 0;                        % Global error flag
errstr = '';
sig = []; tvec = []; delchans=[];

% Get default setup values
	
smoofac = 15;                       % Bernard Joyce's data treatment method is
                                    % fitting a polynomial over 2*SMOOFAC timeframes.
                                    % Default = 15 (should not be changed).
filtfreq = 50;                      % Frequency of the butterworth 4th order digital
                                    % filter used by Christian Deschenaux's channel
                                    % testing and data treatment methods.
                                    % Default = 50 [Hz] (should not be changed).

% ----------------------------------------------------------------------------%
% Signal reading and scaling

                                    
if isempty(btshotnum)               % Test if shotnumber was given.
  
  errflag = 1;
  errstr = (['You didn''t specify a shot number !!!']);
  mdsclose;
  return
  
end
	
mdsopen('tcv_shot',btshotnum);      % Opening the MDS tree for reading.
	
btalldata=mdsdata('\base::bolo:signals');
                              
                                    % Bolometry signal node. 
	   
tau = mdsdata('\base::bolo:tau');   % Loading the bolometer thermal inertia
                                    % constants (for calibration).

if isempty(btalldata)               % Check if the shot exists in MDS.

  errflag = 1;
  errstr = (['Sorry, no data available for shot ',int2str(btshotnum)]);
  mdsclose;
  return
    
end

% *** If TMIN was not specified -> LIUQE times are loaded. *********************************

if isnan(tmin) 
	
  bttimebase=mdsdata(['dim_of(\base::bolo:signals)']);
    
                                     % Loading the NORMAL time base.
                                     
  liuqetbase=mdsdata('dim_of(\results::r_axis)');
    
                                     % Loading the LIUQE time base.
                                     
  mdsclose;

  ii=1;                              % Indicator for NORMAL times nearest to LIUQE times.
  base=[];
  tbase=[];
	   
  for k=1:length(liuqetbase)

    base(ii)=max(find(abs(bttimebase-liuqetbase(k))==min(abs(bttimebase-liuqetbase(k)))));
      
                                     % Index vector of NORMAL times nearest to LIUQE times.
	      
	if ii==1                         % First nearest timeframe.

      tbase(ii)=bttimebase(base(ii));
	  ii=ii+1;
        
    else                             % Following nearest timeframes.
      
      if bttimebase(base(ii))>tbase(ii-1)

        tbase(ii)=bttimebase(base(ii));
		ii=ii+1;
          
      end
        
    end
    
  end

  frametbase=tbase';
	
  [vec,framebase,ib]=intersect(bttimebase,frametbase);
    
                                      % Times matching the time interval of the shots timeframes.
                                      
  [bttotsteps,bttotchan]=size(btalldata);


% *** If TMIN was specified -> NORMAL times are loaded. *************************************
  
else                                
            
  bttimebase=mdsdata(['dim_of(\base::bolo:signals)']);

                                      % Loading the NORMAL time base.
                                     
  mdsclose;

  [bttotsteps,bttotchan]=size(btalldata);

  if isnan(tmax)                      % If TMAX was not specified -> upper time limit is the 
                                      % latest timeframe for the given shot.

    tmax = bttimebase(length(bttimebase)-1);
    
  end
 
  if tmax > bttimebase(length(bttimebase))
   
    tmax = bttimebase(length(bttimebase)-1);
    
  end

  if ~isnan(points)                   % Reducing the number of timeframes to POINTS timeframes
                                      % if points have been submitted.
                                      
	   minstep = min(find(bttimebase>=tmin));
	   maxstep = max(find(bttimebase<=tmax));
	   framebase = round(minstep:(maxstep-minstep)/(points-1):maxstep);
    
  else                                % Taking all the timeframes matching the given time interval.
	   
    framebase = find(bttimebase>=tmin & bttimebase<=tmax);
    points = length(framebase);
    
  end

  frametbase = bttimebase(framebase);
  
end


% ----------------------------------------------------------------------------%
% Channel testing and deleting

if chantest ~= 0

  fprintf('Looking for bad channels:\n\n');
  
end

mdsopen('tcv_shot',btshotnum);

bolosig = mdsdata('\base::bolo:source');         % Reading of the unprocessed (raw) bolometry signals
mdsclose;
	
if ~isempty(delchanuser)                         % Channels deleted by the user (flag DELCHANUSER in
                                                 % 'fabcat.m'

  fprintf('  You have disabled the following channels:\n  ');
  fprintf('%1.0f,', delchanuser');   
  fprintf('\n\n');
  
end


% *** Automated deleting of weird channels using Arno Refke's method **********************
% *** This is the channel consistency check

if ((chantest == 2) | (chantest == 3))	   	

  offset = mean(bolosig(1:80,:));                    % Offset : Mean value of the signals
  bolosig = bolosig - repmat(offset,bttotsteps,1);
  difbolo = diff(bolosig);                           % Slope of the signals.
  gaga1 = find((max(bolosig)-min(bolosig))<0.5);     % Signal value range check.
  gaga2 = find(mean(bolosig)<0);                     % Negative signal check.
  gaga3 = find((max(difbolo)-min(difbolo))>4);       % Signal slope check.
  gaga4 = find(abs(offset)>9);                       % Signal offset check.
  gaga = union(gaga1,gaga2);
  gaga = union(gaga,gaga3);
  gaga = union(gaga,gaga4);

  if ~isempty(gaga)

    fprintf('  Arno Refke''s method disabled the following channels:\n  ');
	fprintf('%1.0f,', gaga');
	fprintf('\n\n');
    
  end
	
  delchans=union(gaga,delchanuser);
 
end

% *** Automated deleting of weird channels using Christian Deschenaux's method *************
% *** This is the signal backfiltering method

if ((chantest == 1) | (chantest == 3))

  p = btalldata';
  amp=.01;
  [b,a]=butter(4,min(1,filtfreq/(2000/2)));         % Design of a  butterworth
                                                    % digital filter of 4th order.

  [py,px]=size(p);
  fltr=zeros(size(bttotchan));

  for i=1:bttotchan

    p(i,:)=filtfilt(b,a,p(i,:));                    % Filtering of the signals.
  	fltr1(i)=mean(p(i,[1:min(100,bttotsteps)]));
  	m=abs(mean(p(i,:)));

  	if m>0                                          % Filtered signal mean value sign check.
      
      fltr2(i)=mean(abs(diff(p(i,:))))/m;           % Filtered signal slope check
      
    else
    
      fltr2(i)=0; 
      
    end;
    
  end;

  fltr1=abs(fltr1)>0.1;
  fltr2=(fltr2<(mean(fltr2)*amp));
  fltr=find(fltr1 | fltr2);
	
  if ~isempty(fltr)

    fprintf('  Christian Deschenaux''s method disabled the following channels:\n  ');
    fprintf('%1.0f,', fltr');
    fprintf('\n\n');
    
  end

  delchans=union(fltr,delchans);
  delchans=union(delchans,delchanuser);
 
end

goodchans=setdiff([1:bttotchan],delchans);


%---------------------------------------------------------------------------%
% Data treatment (calibration) 

if smoothmode
  
  sweep=smoofac;
  
else
    
  sweep=1;
  
end
               	       		
if framebase(1)<sweep+1                 % Time compatibility check,
                                        % to test if methods are applicable.
	 
  errflag = 1;
  errstr = (['The smoothing requires higher start time.']);
  return	   
  
elseif length(framebase) > bttotsteps - sweep
    
  errflag = 1;
  errstr = (['The smoothing requires lower stop time.']);
  return
  
end
	
rawdata=btalldata(framebase,goodchans);  % The ram, uncalibrated signals.

framedata=NaN.*ones(size(rawdata));

% *** No data treatment ****************************************************

if smoothmode == 0

  framedata=1.78e4*rawdata;              % Calculation of the chord brigthness
  
end

% *** Bernard Joye's method **********************************************
% *** This is the second order polynomial fitting method

if smoothmode == 1

  fprintf('  Smoothing using Bernard Joye''s method.\n');
  offset = mean(btalldata(1:80,goodchans));
  basedata = 1.78e4*(btalldata(:,goodchans)...
		      - repmat(offset,bttotsteps,1));
          
                                         % Uncalibrated chord brightness.
                                         
  per_acq=(bttimebase(bttotsteps)-bttimebase(1))/(bttotsteps-1);
  
                                         % mean timestep time.
  xvec=-smoofac:smoofac;                 % Fitting index.
  
  bolocor=NaN.*ones(1,bttotsteps);
     
  for stn=1:length(framebase)            % Fitting timeframe window.

   	k=framebase(stn);
	   yvec=basedata((k-smoofac):(k+smoofac),:);

    for chn=1:length(goodchans)          % Fitting signals using a polinomial
                                         % of second degree taking in account 
                                         % of the bolometers time constants TAU
  		 
      p=polyfit(xvec,yvec(:,chn)',2);
 	    framedata(stn,chn)=p(3)+...
	     tau(goodchans(chn))*p(2)/per_acq;
  
    end
    
  end
  
end

% *** Christian Deschenaux's method **********************************************
% *** This is the backfiltering method

if smoothmode == 2
    
  fprintf('  Smoothing using Christian Deschenaux''s method.\n');
  
  [b,a]=butter(4,min(1,filtfreq/1000));             % Design of a  butterworth
                                                    % digital filter of 4th order.
  unity=ones(size(framebase)); 
  
  difft=bttimebase(framebase+unity)-bttimebase(framebase-unity);
  
                                                    % Neighboring fimeframes 
                                                    % time intervals.
  
  for ch=1:length(goodchans)
	
    pall=filtfilt(b,a,btalldata(:,goodchans(ch)));  % Filtering of the signals.
	   psel=pall(framebase);
	   diffp=pall(framebase+unity)-pall(framebase-unity);
    
                                                    % Derivation of the filtered signals.
	   framedata(:,ch)=1.78e4*...
			      (psel+tau(goodchans(ch))*diffp./difft);
              
                                                    % Calibration using time constants TAU.
              
  end
  
end
      
fprintf('\n');
