function [omega, rho_grid, zet_grid, dV] = ...
    fab3d(Kb1, Kb2, Kb3, Kb4, Kd1, Kd2, Kd3, Kd4, ivert)

% function [omega, rho_grid, zet_grid, dV] = ...
%    fab3d(Kb1, Kb2, Kb3, Kb4, Kd1, Kd2, Kd3, Kd4, ivert)
%
% Calculates a 2D-matrix omega from a 3D grid definded inside
% 'fabsetangle.m'.
% The grid fills approximately a 40 cm thick poloidal slice 
% of the TCV vacuum vessel ('thick': in toroidal direction)
%
% input data: 	Kb1..4: 	midpoints of the edges of the aperture
%		        Kd1..4:		midpoints of the edges of the detector
%				            each K.. has three components:
%				               K..(1): radial rho
%					           K..(2): vertical	zet
%					           K..(3): toroidal	tee
%				               K..1&2: midpoints of edge 'lines' 
%					                   in rho-zet-plane
%				               K..3&4: midpoints of edges
%					                   in tee-zet-plane
%		        ivert:	    a flag, determines if the detector 'looks'
%				            horizontally or vertically
% output data : omega:      The detectors solid angles
%               rho_grid:   The radial coordinates.
%               zet_grid:   The vertical coordinates.
%               dV:         a pixels volume.
%
% This program calls 'fabproject.m'
%
% History:        
%
%  Matthias Anton, CRPP EPFL, 29 of May, 1995 (first release)
%  Christian Schlatter, TPIV CRPP EPFL, February 2002 (comments and renaming)
%  
% Final release version 1.4 (2002/02/07)


% parameter: define the 3D grid
% rho: radial
% zet: vertical
% tee: toroidal coordinate


Rho=88;
brho=2*30;
bzet=2*90;
btee=2*20;

drho=0.5;
dzet=0.5;
dtee=0.5;

dV=dtee*dzet*drho;

rho_grid=Rho-(brho-drho)/2:drho:Rho+(brho-drho)/2;
zet_grid=-(bzet-dzet)/2:dzet:(bzet-dzet)/2;
tee_grid=0:dtee:(btee-dtee)/2;

% get four points of aperture (midpoint of each side)
% 1,2: in the rho-zet plane
% 3,4: in the tee-zet plane

rhob1=Kb1(1);
zetb1=Kb1(2);
teeb1=Kb1(3);

rhob2=Kb2(1);
zetb2=Kb2(2);
teeb2=Kb2(3);

rhob3=Kb3(1);
zetb3=Kb3(2);
teeb3=Kb3(3);

rhob4=Kb4(1);
zetb4=Kb4(2);
teeb4=Kb4(3);

% get four points of detector (midpoint of each side)
% 1,2: in the rho-zet plane
% 3,4: in the tee-zet plane

rhod1=Kd1(1);
zetd1=Kd1(2);
teed1=Kd1(3);

rhod2=Kd2(1);
zetd2=Kd2(2);
teed2=Kd2(3);

rhod3=Kd3(1);
zetd3=Kd3(2);
teed3=Kd3(3);

rhod4=Kd4(1);
zetd4=Kd4(2);
teed4=Kd4(3);

% init omega

omega=zeros(length(zet_grid),length(rho_grid));

% loop over toroidal coordinate 

for it=1:length(tee_grid);

	tee=tee_grid(it);
	rho_shift=Rho-sqrt(Rho^2-tee^2);

	rho=ones(size(zet_grid'))*(rho_grid-rho_shift);
	zet=zet_grid'*ones(size(rho_grid));
    tee=tee*ones(size(zet));

	if ivert

		[rhol1,zetl1,rhol2,zetl2] = ...
		fabproject(rhob1,zetb1,rhob2,zetb2,rhod1,zetd1,rhod2,zetd2,rho,zet);
        
		[teel3,zetl3,teel4,zetl4] = ...
		fabproject(teeb3,zetb3,teeb4,zetb4,teed3,zetd3,teed4,zetd4,tee,zet);

		die=find(rhol1~=99999 & teel3 ~=99999);

		dis_rho=(rhol1(die)+rhol2(die))/2-rho(die);
		dis_zet=(zetl1(die)+zetl2(die)+zetl3(die)+zetl4(die))/4-zet(die);
		dis_tee=(teel3(die)+teel4(die))/2-tee(die);

		surf_rho=(zetl1(die)-zetl2(die)).*(teel3(die)-teel4(die));
		surf_zet=-(rhol1(die)-rhol2(die)).*(teel3(die)-teel4(die));
		surf_tee=(rhol1(die)-rhol2(die)).*(zetl3(die)-zetl4(die));

	else

		[zetl1,rhol1,zetl2,rhol2]=....
		fabproject(zetb1,rhob1,zetb2,rhob2,zetd1,rhod1,zetd2,rhod2,zet,rho);
        
		[teel3,rhol3,teel4,rhol4]=....
		fabproject(teeb3,rhob3,teeb4,rhob4,teed3,rhod3,teed4,rhod4,tee,rho);

		die=find(zetl1~=99999 & teel3 ~=99999);

		dis_rho=(rhol1(die)+rhol2(die)+rhol3(die)+rhol4(die))/4-rho(die);
		dis_zet=(zetl1(die)+zetl2(die))/2-zet(die);
		dis_tee=(teel3(die)+teel4(die))/2-tee(die);

		surf_rho=(zetl1(die)-zetl2(die)).*(teel3(die)-teel4(die));
		surf_zet=-(rhol1(die)-rhol2(die)).*(teel3(die)-teel4(die));
		surf_tee=-(zetl1(die)-zetl2(die)).*(rhol3(die)-rhol4(die));

	end

	dd=(dis_rho.^2+dis_zet.^2+dis_tee.^2).^0.5;
	surfn=abs(surf_rho.*dis_rho + ...
                surf_zet.*dis_zet + ...
                surf_tee.*dis_tee)./dd;
        
	omega(die)=omega(die)...
		+abs(surf_rho.*dis_rho + ...
		surf_zet.*dis_zet + ...
		surf_tee.*dis_tee)./ dd.^3;

	if it==1;

        omega0=omega;
        
    end

end

omega=2*omega-omega0;

