function [OMEGA, rho_grid, zet_grid] = fabsetsolidangle

% function [OMEGA, rho_grid, zet_grid] = fabsetsolidangle
% 
% 'fabsetsolidangle.m' calculates the individual bolometers solid angles and stores
% them to files called 'fabsolidangle_#.mat' in the current working directory.
% 
% The 'fabsolidangle_#.mat' files are used by the routine 'fabsolidangle.m' which helps
% to calculate the T-matrix (initiated by 'fabsett.m')
%
% This program calls 'fab3d.m'
%
% History:        
%
%  Matthias Anton, CRPP EPFL, 29 of May, 1995 (first release)
%  Jan Mlynar, CRPP EPFL, 20 of August, 1998 (adaption for bolometry)
%  Christian Schlatter, TPIV CRPP EPFL, February 2002 (comments and renaming)
%  
% Final release version 1.4 (2002/02/07)

global Kb1 Kb2 Kb3 Kb4 Kd1 Kd2 Kd3 Kd4

debut=cputime;

disp(' ') 
disp('Solid angle calculator for bolometry.');
disp('Final release 1.4')
disp('Latest modification on 2002/02/07.') 
disp(' '); 

% Detector parameters

cw=1;                         % Detector numbers cw = 1 : clockwise cw = 0 : ccw
fans=[1 1 1 1 1 1 1 1];       % Camera switch
vangle=[90 0 0 0 0 0 0 -90];  % Angle of detector surface normal
xpos=[88 123.5 123.5 123.5 123.5 123.5 123.5 88]; 
                              % x position of the diaphragmas in [cm]
ypos=[81.5 45.5 45.5 -.25 -.25 -46 -46 -81.5];
                              % y position of the diaphragmas in [cm]
  
% Position of the detectors

rdet=[0.862,0.867,0.8721,0.8771,0.8829,0.8879,0.893,0.898,...
1.278,1.2806,1.2833,1.2861,1.2883,1.2893,1.2902,1.2912];

rdet(17:24)=fliplr(rdet(9:16));
rdet(25:40)=rdet(9:24);
rdet(41:56)=rdet(9:24);
rdet(57:64)=fliplr(rdet(1:8));

zdet=[0.9057,0.9063,0.9069,0.9075];
zdet(5:8)=fliplr(zdet(1:4));
zdet(9:24)=[0.4916,0.4873,0.483,0.4787,0.473,0.468,0.4631,0.4581,...
0.4519,0.4469,0.442,0.437,0.4313,0.427,0.4227,0.4184];
zdet(25:40)=zdet(9:24)-0.4575;
zdet(41:56)=zdet(9:24)-0.915;
zdet(57:64)=-zdet(1:8);

xdet=rdet*100; ydet=zdet*100;

d1=1.5/10;			            % Detector width      in cm
d2=4/10;			            % Detector length     in cm
b1=[2.6 2.2 2.2 2.2 2.2 2.2 2.2 2.6]/5;			 
   				                % Aperture width      in cm (pol.)
b2=[10 8 8 8 8 8 8 10]/5;			 
				                % Aperture length     in cm (tor.)

% Calculation of aperture and detector positions

nact=sum(fans);
iact=find(fans);
ndet=8;
ncam=8;

% Aperture 1 (poloidally limiting)

xap=ones(ndet,1)*xpos;
xap=xap(:)';
yap=ones(ndet,1)*ypos;
yap=yap(:)';

% Detectors

dxd1=xdet-xap;
dyd1=ydet-yap;

ivert=[1:8,57:64]';
ihori=[9:56]';

dxd2=dxd1; dyd2=dyd1;   % Supp. da = da2 i.e. diaphragma-array distance is 		
			            % equal in poloidal (da) and toroidal (da2) directions
xap2=xdet-dxd2;
yap2=ydet-dyd2;

xyw=ones(ndet,1)*b1/2;
xyw=xyw(:)';

idxw(ivert)=ones(size(ivert));
idyw(ihori)=ones(size(ihori));
idyw(64)=0;

dxw=xyw;
dyw=xyw;
dzw=ones(ndet,1)*b2(iact)/2;
dzw=dzw(:)';

% x, y: poloidal cross section; z: 'toroidal' coordinate
% Transform to the 'format' used by 'fab3d.m':
% 1, 2 : midpoints on the straight lines which limit the detector poloidally
% 3, 4 ; midpoints on the straight lines which limit the detector toroidally

Kb1=[xap-dxw;yap-dyw;zeros(size(xap))];
Kb2=[xap+dxw;yap+dyw;zeros(size(xap))];
Kb3=[xap2;yap2;-dzw];
Kb4=[xap2;yap2;dzw];

Kd1=[xdet-idxw*d1/2;ydet-idyw*d1/2;zeros(size(xap))];
Kd2=[xdet+idxw*d1/2;ydet+idyw*d1/2;zeros(size(xap))];
Kd3=[xdet;ydet;-d2*ones(size(xdet))/2];
Kd4=[xdet;ydet;d2*ones(size(xdet))/2];

% Get the detector names

detn=[];

for j=1:length(fans)

	if fans(j)

        detn=[detn,(j-1)*8+1:j*8];
        
    end

end

% Calculate the solid angle for the whole grid and every detector

for kk=1:length(xap)

    disp('');
    disp(['  Calculation of the solid angle for detector ', int2str(kk)]);

	[omega, rho_grid, zet_grid, dV] = ...
	fab3d(Kb1(:,kk), Kb2(:,kk), Kb3(:,kk), Kb4(:,kk), ...
		  Kd1(:,kk), Kd2(:,kk), Kd3(:,kk), Kd4(:,kk), idxw(kk));

	ici=find(omega);
    rhodum=ones(size(zet_grid'))*rho_grid;
	zetdum=zet_grid'*ones(size(rho_grid));
    rhoici=rhodum(ici);
	zetici=zetdum(ici);
    omeici=omega(ici);
	i_detec=1;

    eval(['save fabsolidangle_',int2str(detn(kk)),......
	'.mat ici rho_grid zet_grid rhoici zetici omeici dV i_detec']);
 
end

OMEGA=[];

disp(' ');
disp('Successfully finished');
disp(' ');

time_end = (cputime - debut)/60;
disp(['CPU runtime: ' num2str(time_end),' min.']);
disp(' ');
