function [TT, numdet] = fabstandard(xchord, ychord, ...
    xmin, xmax, ymin, ymax, nx, ny);

% function [TT, numdet] = fabstandard(xchord, ychord, ...
%    xmin, xmax, ymin, ymax, nx, ny);
%
% A fast algorithm to calculate the lengths of the chords given by
% xchord, ychord in pixels of a grid specified by the other inputs.
%
% Determination only based on geometry
%
% Inputs 
%
%	xchord, ychord: endpoints of lines of sight, size [2 x nl]
% 	xmin ... ymax:	corners of pixel grid
%	nx, ny:		    number of pixels horizontal,vertical
%		
% Output
%
%	TT:		transfermatrix [length(numdet) x nx*ny],
%			TT(l,i) is the length of chord l in pixel i
%	numdet:	numbers of 'active' lines of sight, usually
%			length(numdet) < nl
% 
% History:        
%
%  Matthias Anton, CRPP EPFL, 9 of August 1994 (first release)
%  Matthias Anton, CRPP EPFL, 2nd of December 1994
%  
% Final release version 1.4 (2002/02/07)

[dummy,nl]=size(xchord);

dx=(xmax-xmin)/nx;
dy=(ymax-ymin)/ny;
xgrid=xmin:dx:xmax;
ygrid=ymin:dy:ymax;
xpix=xmin+dx/2:dx:xmax-dx/2;
ypix=ymin+dy/2:dy:xmax-dy/2;
numpix=reshape(1:nx*ny,ny,nx);

for k=1:nl

  c=polyfit(xchord(:,k),ychord(:,k),1);
  m(k)=c(1);b(k)=c(2);
  
end

% crossing with vertical lines of grid

Ysec=zeros(nl,nx+1);
X=ones(nl,1)*xgrid;
Ysec=m'*xgrid+b'*ones(1,nx+1);

% crossings with horizontal lines of grid

Xsec=zeros(nl,ny+1);
Y=ones(nl,1)*ygrid;
Xsec=(ones(nl,1)*ygrid-b'*ones(1,ny+1))./(m'*ones(1,ny+1));

% matrices with x and y coordinates of all crossings with meshgrid

XX=[X,Xsec];
YY=[Ysec,Y];

% sorting in ascending order of X

for k=1:nl

  [XX(k,:),ind]=sort(XX(k,:));
  YY(k,:)=YY(k,ind);
  
end

dX=[diff(XX')' zeros(nl,1)];
dY=[diff(YY')' zeros(nl,1)];

% trying to find out the pixel numbers

XXnum=zeros(size(XX));
YYnum=zeros(size(YY));

for k=1:nx

  hx=find(XX+dX/2>xgrid(k) & XX+dX/2<xgrid(k+1) & dX>0);
  XXnum(hx)=k*ones(size(hx));
  
end

for k=1:ny

  hy=find( (YY+dY/2>ygrid(k) & YY + dY/2 < ygrid(k+1)) & dX>0 );
  YYnum(hy)=k*ones(size(hy));
  
end
 
seglength=sqrt(dX.^2+dY.^2);
TT=zeros(nl,nx*ny);

for k=1:nl
    
  cols=find((YYnum(k,:)~=0)&(XXnum(k,:)~=0)&...
        (seglength(k,:)<=sqrt(dx^2+dy^2)));
  dummy=diag( numpix(YYnum(k,cols),XXnum(k,cols)) );
  
  if ~isempty(dummy)

    TT(k,dummy(:))=seglength(k,cols);
    
  end
  
end

numdet=find(sum(TT'));
TT=TT(numdet,:);